@@ -75,10 +75,11 @@
 #define MENU_PATH_SPATIAL_BOOKMARKS_PLACEHOLDER	"/MenuBar/Other Menus/Places/Bookmarks Placeholder"
 
 struct _NautilusSpatialWindowDetails {
-        GtkActionGroup *spatial_action_group; /* owned by ui_manager */
-	char *last_geometry;	
-        guint save_geometry_timeout_id;	  
-	
+	GtkActionGroup *spatial_action_group; /* owned by ui_manager */
+	char *last_geometry;
+	guint save_geometry_timeout_id;
+
+	gboolean saved_data_on_close;
 	GtkWidget *content_box;
 	GtkWidget *location_button;
 	GtkWidget *location_label;
@@ -153,7 +154,7 @@ nautilus_spatial_window_configure_event (GtkWidget *widget,
 		window->details->last_geometry = geometry_string;
 
 		window->details->save_geometry_timeout_id = 
-			g_timeout_add (1000, save_window_geometry_timeout, window);
+			g_timeout_add_seconds (1, save_window_geometry_timeout, window);
 	}
 	
 	return FALSE;
@@ -189,7 +190,7 @@ nautilus_spatial_window_state_event (GtkWidget *widget,
 	NautilusFile *viewed_file;
 
 	window = NAUTILUS_WINDOW (widget);
-	slot = window->details->active_slot;
+	slot = window->details->active_pane->active_slot;
 	viewed_file = slot->viewed_file;
 
 	if (!NAUTILUS_IS_DESKTOP_WINDOW (widget)) {
@@ -260,7 +261,7 @@ nautilus_spatial_window_save_geometry (NautilusWindowSlot *slot)
 	NautilusFile *viewed_file;
 	char *geometry_string;
 
-	window = NAUTILUS_WINDOW (slot->window);
+	window = NAUTILUS_WINDOW (slot->pane->window);
 
 	viewed_file = slot->viewed_file;
 
@@ -288,9 +289,10 @@ nautilus_spatial_window_save_scroll_position (NautilusWindowSlot *slot)
 	NautilusWindow *window;
 	char *scroll_string;
 
-	window = NAUTILUS_WINDOW (slot->window);
+	window = NAUTILUS_WINDOW (slot->pane->window);
 
-	if (slot->content_view == NULL) {
+	if (slot->content_view == NULL ||
+	    slot->viewed_file == NULL) {
 		return;
 	}
 	
@@ -309,7 +311,11 @@ nautilus_spatial_window_save_show_hidden_files_mode (NautilusWindowSlot *slot)
 	char *show_hidden_file_setting;
 	NautilusWindowShowHiddenFilesMode mode;
 
-	window = NAUTILUS_WINDOW (slot->window);
+	if (slot->viewed_file == NULL) {
+		return;
+	}
+	
+	window = NAUTILUS_WINDOW (slot->pane->window);
 
 	mode = NAUTILUS_WINDOW (window)->details->show_hidden_files_mode;
 	if (mode != NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT) {
@@ -439,38 +445,67 @@ real_set_allow_up (NautilusWindow *window, gboolean allow)
 }
 
 static NautilusWindowSlot *
-real_open_slot (NautilusWindow *window,
+real_open_slot (NautilusWindowPane *pane,
 		NautilusWindowOpenSlotFlags flags)
 {
 	NautilusWindowSlot *slot;
 	GList *slots;
 
-	g_assert (nautilus_window_get_active_slot (window) == NULL);
+	g_assert (nautilus_window_get_active_slot (pane->window) == NULL);
 
-	slots = nautilus_window_get_slots (window);
+	slots = nautilus_window_get_slots (pane->window);
 	g_assert (slots == NULL);
 	g_list_free (slots);
 
 	slot = g_object_new (NAUTILUS_TYPE_WINDOW_SLOT, NULL);
-	slot->window = window;
-	gtk_container_add (GTK_CONTAINER (NAUTILUS_SPATIAL_WINDOW (window)->details->content_box),
+	slot->pane = pane;
+	gtk_container_add (GTK_CONTAINER (NAUTILUS_SPATIAL_WINDOW (pane->window)->details->content_box),
 			   slot->content_box);
 	gtk_widget_show (slot->content_box);
 	return slot;
 }
 
 static void
-real_close_slot (NautilusWindow *window,
-		 NautilusWindowSlot *slot)
+save_spatial_data (NautilusWindowSlot *slot)
 {
-	g_assert (g_list_length (window->details->slots) == 1);
-
 	nautilus_spatial_window_save_geometry (slot);
 	nautilus_spatial_window_save_scroll_position (slot);
 	nautilus_spatial_window_save_show_hidden_files_mode (slot);
+}
+
+static void
+real_close_slot (NautilusWindowPane *pane,
+		 NautilusWindowSlot *slot)
+{
+	/* Save spatial data for close if we didn't already */
+	if (!NAUTILUS_SPATIAL_WINDOW (pane->window)->details->saved_data_on_close) {
+		save_spatial_data (slot);
+	}
 
 	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS,
-			 close_slot, (window, slot));
+			 close_slot, (pane, slot));
+}
+
+static void
+real_window_close (NautilusWindow *window)
+{
+	NautilusWindowSlot *slot;
+
+	/* We're closing the window, save the geometry. */
+	/* Note that we do this in window close, not slot close, because slot
+	 * close is too late, by then the widgets have been unrealized.
+	 * This is for the close by WM case, if you're closing via Ctrl-W that
+	 * means we close the slots first and this is not an issue */
+	if (window->details->active_pane != NULL &&
+	    window->details->active_pane->active_slot != NULL) {
+		slot = window->details->active_pane->active_slot;
+
+		save_spatial_data (slot);
+		NAUTILUS_SPATIAL_WINDOW (window)->details->saved_data_on_close = TRUE;
+	}
+
+	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS,
+			 close, (window));
 }
 
 static void
@@ -483,7 +518,7 @@ location_menu_item_activated_callback (GtkWidget *menu_item,
 	GFile *dest;
 	GdkEvent *event;
 
-	slot = window->details->active_slot;
+	slot = window->details->active_pane->active_slot;
 
 	location = nautilus_window_slot_get_location_uri (slot);
 	current = g_file_new_for_uri (location);
@@ -553,7 +588,7 @@ got_file_info_for_location_menu_callback (NautilusFile *file,
 
 	if (pixbuf != NULL) {
 		icon = gtk_image_new_from_pixbuf (pixbuf);
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 	} else {
 		icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
 	}
@@ -609,7 +644,7 @@ location_button_pressed_callback (GtkWidget      *widget,
 {
 	NautilusView *view;
 
-	view = window->details->active_slot->content_view;
+	view = window->details->active_pane->active_slot->content_view;
 
 	if (event->button == 3 && view != NULL) {
 		nautilus_view_pop_up_location_context_menu (view, event, NULL);
@@ -629,7 +664,7 @@ location_button_clicked_callback (GtkWidget             *widget,
 	GFile *child_uri;
 	GMainLoop *loop;
 
-	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
 
 	popup = gtk_menu_new ();
 	first_item = NULL;
@@ -649,6 +684,7 @@ location_button_clicked_callback (GtkWidget             *widget,
 
 		name = nautilus_file_get_display_name (file);
 		menu_item = gtk_image_menu_item_new_with_label (name);
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menu_item), TRUE);
 		g_free (name);
 
 		if (first_item == NULL) {
@@ -708,7 +744,8 @@ location_button_clicked_callback (GtkWidget             *widget,
 	g_main_loop_run (loop);
 	gtk_grab_remove (popup);
 	g_main_loop_unref (loop);
- 	gtk_object_sink (GTK_OBJECT (popup));
+	g_object_ref_sink (popup);
+	g_object_unref (popup);
 }
 
 static int
@@ -719,7 +756,7 @@ get_dnd_icon_size (NautilusSpatialWindow *window)
 	NautilusZoomLevel zoom_level;
 
 	parent = NAUTILUS_WINDOW(window);
-	view = parent->details->active_slot->content_view;
+	view = parent->details->active_pane->active_slot->content_view;
 
 	if (view == NULL) {
 		return NAUTILUS_ICON_SIZE_STANDARD;
@@ -738,7 +775,7 @@ location_button_drag_begin_callback (GtkWidget             *widget,
 	NautilusWindowSlot *slot;
 	GdkPixbuf *pixbuf;
 
-	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
 
 	pixbuf = nautilus_file_get_icon_pixbuf (slot->viewed_file,
 						get_dnd_icon_size (window),
@@ -766,7 +803,7 @@ get_data_binder (NautilusDragEachSelectedItemDataGet iteratee,
 	g_assert (NAUTILUS_IS_SPATIAL_WINDOW (iterator_context));
 	window = NAUTILUS_SPATIAL_WINDOW (iterator_context);
 
-	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
 
 	location = nautilus_window_slot_get_location_uri (slot);
 	icon_size = get_dnd_icon_size (window);
@@ -822,7 +859,7 @@ nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
 
 			if (pixbuf != NULL) {
 				gtk_image_set_from_pixbuf (GTK_IMAGE (window->details->location_icon),  pixbuf);
-				gdk_pixbuf_unref (pixbuf);
+				g_object_unref (pixbuf);
 			} else {
 				gtk_image_set_from_stock (GTK_IMAGE (window->details->location_icon),
 							  GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
@@ -907,6 +944,14 @@ static const GtkActionEntry spatial_entries[] = {
                                  G_CALLBACK (action_search_callback) },
 };
 
+static const char* icon_entries[] = {
+	"/MenuBar/Other Menus/Places/Home",
+	"/MenuBar/Other Menus/Places/Go to Computer",
+	"/MenuBar/Other Menus/Places/Go to Templates",
+	"/MenuBar/Other Menus/Places/Go to Trash",
+	"/MenuBar/Other Menus/Places/Go to Network"
+};
+
 static void
 nautilus_spatial_window_init (NautilusSpatialWindow *window)
 {
@@ -917,11 +962,28 @@ nautilus_spatial_window_init (NautilusSpatialWindow *window)
 	GtkUIManager *ui_manager;
 	GtkTargetList *targets;
 	const char *ui;
+	int i;
+	GtkWidget *menuitem;
+	NautilusWindow *win;
+	NautilusWindowPane *pane;
 
 	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window,
 						       NAUTILUS_TYPE_SPATIAL_WINDOW,
 						       NautilusSpatialWindowDetails);
 
+	win = NAUTILUS_WINDOW (window);
+
+	gtk_table_attach (GTK_TABLE (win->details->table),
+			  win->details->statusbar,
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               5, 6,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
+			  0,                                  0);
+	gtk_widget_show (win->details->statusbar);
+
+	pane = nautilus_window_pane_new (win);
+	win->details->panes = g_list_prepend (win->details->panes, pane);
+
 	window->affect_spatial_window_on_next_location_change = TRUE;
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -1008,6 +1070,12 @@ nautilus_spatial_window_init (NautilusSpatialWindow *window)
 	
 	ui = nautilus_ui_string_get ("nautilus-spatial-window-ui.xml");
 	gtk_ui_manager_add_ui_from_string (ui_manager, ui, -1, NULL);
+
+	for (i = 0; i < G_N_ELEMENTS (icon_entries); i++) {
+		menuitem = gtk_ui_manager_get_widget (ui_manager, icon_entries[i]);
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menuitem), TRUE);
+	}
+	nautilus_window_set_active_pane (win, pane);
 }
 
 static void
@@ -1039,6 +1107,7 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 		real_set_allow_up;
 
 	NAUTILUS_WINDOW_CLASS (class)->open_slot = real_open_slot;
+	NAUTILUS_WINDOW_CLASS (class)->close = real_window_close;
 	NAUTILUS_WINDOW_CLASS (class)->close_slot = real_close_slot;
 
 	binding_set = gtk_binding_set_by_class (class);
