@@ -38,7 +38,6 @@
 #include <eel/eel-gtk-extensions.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-macros.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-marshal.h>
@@ -89,9 +88,9 @@ static const int nautilus_zoom_control_accessible_action_signals[] = {
 };
 
 static const char * const nautilus_zoom_control_accessible_action_descriptions[] = {
-	N_("Show the contents in more detail"),
-	N_("Show the contents in less detail"),
-	N_("Show in the default detail level"),
+	N_("Increase the view size"),
+	N_("Decrease the view size"),
+	N_("Use the normal view size")
 };
 
 static GtkMenu *create_zoom_menu (NautilusZoomControl *zoom_control);
@@ -103,15 +102,14 @@ static GType nautilus_zoom_control_accessible_get_type (void);
 
 #define NUM_ACTIONS ((int)G_N_ELEMENTS (nautilus_zoom_control_accessible_action_names))
 
-GNOME_CLASS_BOILERPLATE (NautilusZoomControl, nautilus_zoom_control,
-			 GtkHBox, GTK_TYPE_HBOX)
+G_DEFINE_TYPE (NautilusZoomControl, nautilus_zoom_control, GTK_TYPE_HBOX);
 
 static void
 nautilus_zoom_control_finalize (GObject *object)
 {
 	g_list_free (NAUTILUS_ZOOM_CONTROL (object)->details->preferred_zoom_levels);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (nautilus_zoom_control_parent_class)->finalize (object);
 }
 
 static void 
@@ -268,7 +266,7 @@ label_style_set_callback (GtkWidget *label,
 }		  
 
 static void
-nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
+nautilus_zoom_control_init (NautilusZoomControl *zoom_control)
 {
 	GtkWidget *image;
 	int i;
@@ -292,6 +290,8 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_out), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_out),
 			       GTK_RELIEF_NONE);
+	gtk_widget_set_tooltip_text (zoom_control->details->zoom_out,
+				     _("Decrease the view size"));
 	g_signal_connect (G_OBJECT (zoom_control->details->zoom_out),
 			  "clicked", G_CALLBACK (zoom_out_clicked),
 			  zoom_control);
@@ -303,6 +303,8 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_button), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_button),
 			       GTK_RELIEF_NONE);
+	gtk_widget_set_tooltip_text (zoom_control->details->zoom_button,
+				     _("Use the normal view size"));
 			       
 	gtk_widget_add_events (GTK_WIDGET (zoom_control->details->zoom_button),
 			       GDK_BUTTON_PRESS_MASK
@@ -339,6 +341,8 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_in), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_in),
 			       GTK_RELIEF_NONE);
+	gtk_widget_set_tooltip_text (zoom_control->details->zoom_in,
+				     _("Increase the view size"));
 	g_signal_connect (G_OBJECT (zoom_control->details->zoom_in),
 			  "clicked", G_CALLBACK (zoom_in_clicked),
 			  zoom_control);
@@ -941,3 +945,11 @@ nautilus_zoom_control_accessible_get_type (void)
 
 	return type;
 }
+
+void
+nautilus_zoom_control_set_active_appearance (NautilusZoomControl *zoom_control, gboolean is_active)
+{
+	gtk_widget_set_sensitive (gtk_bin_get_child (GTK_BIN (zoom_control->details->zoom_in)), is_active);
+	gtk_widget_set_sensitive (gtk_bin_get_child (GTK_BIN (zoom_control->details->zoom_out)), is_active);
+	gtk_widget_set_sensitive (zoom_control->details->zoom_label, is_active);
+}
